/** Metric types supported by StatsD. */
export declare enum EMetricType {
    Increment = 0,
    Decrement = 1,
    Gauge = 2,
    Timing = 3,
    Histogram = 4
}
/** Metric tags. */
export interface IMetricTags extends Object {
    [key: string]: any;
}
/** Abstract metric class. */
export declare abstract class Metric {
    /** Send increment counter metric. */
    increment(name: string, value?: number, tags?: IMetricTags, ...args: any[]): void;
    /** Send decrement counter metric. */
    decrement(name: string, value?: number, tags?: IMetricTags, ...args: any[]): void;
    /** Send value metric. */
    gauge(name: string, value: number, tags?: IMetricTags, ...args: any[]): void;
    /** Send time metric in milliseconds. */
    timing(name: string, value: number | Date, tags?: IMetricTags, ...args: any[]): void;
    /** Send distribution value metric. */
    histogram(name: string, value: number, tags?: IMetricTags, ...args: any[]): void;
    /** Metric method provided by implementor. */
    protected abstract metric(type: EMetricType, name: string, value: any, tags: IMetricTags, ...args: any[]): void;
}
